/* file "porky.cc" */


/*
       Copyright (c) 1998 Stanford University

       All rights reserved.

       This software is provided under the terms described in
       the "suif_copyright.h" include file.
*/

#include <suif_copyright.h>


/*
      This is the main implementation file of porky, a library of core
      passes for converting between different dialects of SUIF.
*/


#define _MODULE_ "libporky"

#pragma implementation "porky.h"


#include <suif.h>
#include <suifpasses.h>
#include "make_empty_file_set_symbol_table.h"
#include "dismantle.h"


extern "C" void enter_porky(int *argc, char *argv[])
  {
    static boolean init_done = FALSE;

    if (init_done)
        return;
    init_done = TRUE;
    enter_suif(argc, argv);
    enter_suifpasses(argc, argv);

    register_suif_pass(new make_empty_file_set_symbol_table_pass());

    register_suif_pass(new dismantle_if_statements_pass());
    register_suif_pass(new dismantle_while_statements_pass());
    register_suif_pass(new dismantle_do_while_statements_pass());
    register_suif_pass(new dismantle_for_statements_pass());
    register_suif_pass(new dismantle_scope_statements_pass());
    register_suif_pass(new dismantle_multi_way_branch_statements_pass());
    register_suif_pass(new dismantle_select_instructions_pass());
    register_suif_pass(new dismantle_array_reference_instructions_pass());
    register_suif_pass(new dismantle_field_access_instructions_pass());
    register_suif_pass(new dismantle_extract_fields_instructions_pass());
    register_suif_pass(new dismantle_set_fields_instructions_pass());
    register_suif_pass(new dismantle_extract_elements_instructions_pass());
    register_suif_pass(new dismantle_set_elements_instructions_pass());
    register_suif_pass(new dismantle_bit_size_of_instructions_pass());
    register_suif_pass(new dismantle_byte_size_of_instructions_pass());
    register_suif_pass(new dismantle_bit_alignment_of_instructions_pass());
    register_suif_pass(new dismantle_byte_alignment_of_instructions_pass());
    register_suif_pass(new dismantle_bit_offset_of_instructions_pass());
    register_suif_pass(new dismantle_byte_offset_of_instructions_pass());
    register_suif_pass(new dismantle_sc_and_instructions_pass());
    register_suif_pass(new dismantle_sc_or_instructions_pass());
    register_suif_pass(new dismantle_sc_select_instructions_pass());
    register_suif_pass(new dismantle_load_value_block_instructions_pass());
    register_suif_pass(new dismantle_multi_way_branch_instructions_pass());
  }

extern "C" void exit_porky(void)
  {
    /* empty */
  }
